#!/bin/sh

sender_list(){
  local select="$1" n name
  [ -d senders/ ] && for n in '' senders/*; do
    [ "$n" ] && name="$(sed q "$n" |HTML)"
    [ "${n#senders/}" = "$select" ] \
    && printf '<option value="%s" selected=selected>%s</option>' "${n#senders/}" "$name" \
    || printf '<option value="%s">%s</option>' "${n#senders/}" "$name"
  done
}

client_list(){
  local select="$1" n address hourly name
  [ -d clients/ ] && for n in '' clients/*; do
    [ "$n" ] && read -r address hourly <"$n"
    name="$(UNSTRING "${address#address=}" |sed q |HTML)"
    [ "${n#clients/}" = "$select" ] \
    && printf '<option value="%s" selected=selected>%s</option>' "${n#clients/}" "$name" \
    || printf '<option value="%s">%s</option>' "${n#clients/}" "$name"
  done
}

list_invoices(){
  [ -d invoices/ ] && for i in invoices/*; do
    read -r sender client date number vat vatrate<<-EOF
	$(sed q "$i")
	EOF
    [ -f "senders/${sender#sender=}" ] \
    && sender="$(sed q "senders/${sender#sender=}")" \
    || sender="(unset)"
    [ ! -f "clients/${client#client=}" ] \
    && client="(unset)" \
    || read -r client hourly <"clients/${client#client=}"
    [ "${date#date=}" -ge 0 ] 2>&- \
    && date="$(date -d "@${date#date=}" +%x)" \
    || date="(unset)"

    printf '[div .invoice
      [h2
          %s]
      [label From:] %s [label To:] %s [label on] %s
      [a href="/invoice/%s" Edit]
    ]' "$(UNSTRING "${number#number=}" |HTML)" \
       "$(HTML "$sender")" \
       "$(UNSTRING "${client#address=}" |sed q |HTML)" "$(HTML "$date")" \
       "$(HTML ${i#invoices/})"
  done
}

edit_invoice(){
  id="$1"
  if [ -f "invoices/$id" ]; then
    read -r sender client date number vat vatrate<<-EOF
	$(sed q "invoices/$id")
	EOF
  fi

  [ "${date#date=}" -ge 0 ] 2>&- \
  && date="$(date -d "@${date#date=}" +%F)" \
  || date="$(date +%F)"
  [ "${number#number=}" ] || number="number=$(date +%s)"
  [ "${vatrate#vatrate=}" -ge 0 ] 2>&- || vatrate="vatrate=19"

  cat <<-EOF 
	[form method="POST" action="/update_invoice"
	  [hidden "id" "$(HTML "$id")"]

	  [label Sender:]
	  [select name=sender
	    $(sender_list "${sender#sender=}")
	  ]

	  [label Client:]
	  [select name=client
	    $(client_list "${client#client=}")
	  ]
	
          [label for=number Invoice Number:]
          [input #number name=number value="$(UNSTRING "${number#number=}" |HTML)"]

          [label for=date Date:]
          [input #date name=date value="${date}" placeholder="YYYY-MM-TT"]

          [radio "vat" "smallbusiness" #vatsb $([ "${vat#vat=}" = smallbusiness ] && printf checked) ]
          [label for=vatsb Small business exemption from VAT]
          [radio "vat" "nett" #vatnett $([ "${vat#vat=}" = nett ] && printf checked)]
          [label for=vatnett Nett]
          [radio "vat" "gross" #vatgross $([ "${vat#vat=}" = gross ] && printf checked)]
          [label for=vatgross Gross]
          [label for=vatrate VAT Rate: [input type=number name="vatrate" value="${vatrate#vatrate=}"]% ]

          [table
            [tr [th Date] [th Work] [th Hours] [th Price] ]
$({ sed 1d "invoices/$id"; printf 'time= work= hours=\n'; } \
    | while read -r time work hours; do
      hours="$(UNSTRING "${hours#hours=}" |grep -m1 -xE '[0-9]+' || printf 0)"
      printf '[tr
              [td [textarea name=date
%s] ]
              [td [textarea name=work
%s] ]
              [td [input type=number name=hours value="%s"] ]
              [td %s]
     ]' "$(UNSTRING "${time#time=}" |HTML)" \
        "$(UNSTRING "${work#work=}" |HTML)" \
        "$hours" "$((hours * hourly)) €"
    done
)
          ]
	  [submit "update" "update" Update]
	]
	EOF
}

update_invoice(){
  id="$(POST id |checkid)"
  if [ "$(POST update)" = update -a "$id" ]; then
    mkdir -p invoices
    printf 'sender=%s	client=%s	date=%s	number=%s	vat=%s	vatrate=%s\n' \
      "$(POST sender)" "$(POST client)" \
      "$(date -d "$(POST date)" +%s)" \
      "$(POST number |STRING)" \
      "$(POST vat |grep -m1 -xE 'smallbusiness|gross|nett')" \
      "$(POST vatrate |grep -m1 -xE '[0-9]+')" \
    >"invoices/$id"
  fi
}
